function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React, { Component, PropTypes } from 'react';
import { List } from 'react-virtualized';
import classNames from 'classnames';
import { keyCodes } from '../utils';
import addYears from 'date-fns/add_years';
import setYear from 'date-fns/set_year';
import isAfter from 'date-fns/is_after';
import isBefore from 'date-fns/is_before';
var styles = {
  'root': 'Cal__Years__root',
  'list': 'Cal__Years__list',
  'center': 'Cal__Years__center',
  'year': 'Cal__Years__year',
  'active': 'Cal__Years__active',
  'currentYear': 'Cal__Years__currentYear'
};

var Years = function (_Component) {
  _inherits(Years, _Component);

  function Years(props) {
    _classCallCheck(this, Years);

    var _this = _possibleConstructorReturn(this, _Component.call(this, props));

    _this.state = {
      selectedYear: props.selectedDate ? props.selectedDate.getFullYear() : new Date().getFullYear()
    };
    return _this;
  }

  Years.prototype.componentDidMount = function componentDidMount() {
    var list = this.refs.list;
    var grid = list && list.refs.Grid;

    this.scrollEl = grid && grid.refs.scrollingContainer;
  };

  Years.prototype.handleClick = function handleClick(year, e) {
    var _props = this.props,
        hideYearsOnSelect = _props.hideYearsOnSelect,
        scrollToDate = _props.scrollToDate,
        selectedDate = _props.selectedDate,
        setDisplay = _props.setDisplay;

    var date = selectedDate || new Date();
    var newDate = setYear(date, year);

    this.selectDate(newDate, e, !hideYearsOnSelect);
    scrollToDate(newDate, -40);

    if (hideYearsOnSelect) {
      setDisplay('days');
    }
  };

  Years.prototype.selectDate = function selectDate(date, e) {
    var updateState = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
    var shouldHeaderAnimate = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
    var _props2 = this.props,
        minDate = _props2.minDate,
        maxDate = _props2.maxDate,
        onDaySelect = _props2.onDaySelect;


    if (!isBefore(date, minDate) && !isAfter(date, maxDate)) {
      if (updateState) {
        this.setState({
          selectedYear: date.getFullYear()
        });
      }

      onDaySelect(date, e, shouldHeaderAnimate);
    }
  };

  Years.prototype.handleKeyDown = function handleKeyDown(e) {
    var _props3 = this.props,
        scrollToDate = _props3.scrollToDate,
        setDisplay = _props3.setDisplay,
        selectedDate = _props3.selectedDate;
    var selectedYear = this.state.selectedYear;

    var delta = 0;

    switch (e.keyCode) {
      case keyCodes.enter:
      case keyCodes.escape:
        setDisplay('days');
        scrollToDate(selectedDate || new Date(selectedYear, 0), -40);
        return;
      case keyCodes.down:
        delta = +1;
        break;
      case keyCodes.up:
        delta = -1;
        break;
      default:
        delta = 0;
    }

    if (delta) {
      if (!selectedDate) selectedDate = new Date(selectedYear, 0);

      var newSelectedDate = addYears(selectedDate, delta);
      this.selectDate(newSelectedDate, e);
    }
  };

  Years.prototype.render = function render() {
    var _this2 = this;

    var _props4 = this.props,
        height = _props4.height,
        selectedDate = _props4.selectedDate,
        theme = _props4.theme,
        width = _props4.width;
    var selectedYear = this.state.selectedYear;

    var today = new Date();
    var currentYear = today.getFullYear();
    var years = this.props.years.slice(0, this.props.years.length);

    // Add spacer rows at the top and bottom
    years.unshift(null);
    years.push(null);

    var selectedYearIndex = years.indexOf(selectedYear);
    var rowHeight = 50;
    var containerHeight = years.length * rowHeight < height + 50 ? years.length * rowHeight : height + 50;

    return React.createElement(
      'div',
      {
        className: styles.root,
        style: { color: theme.selectionColor, height: height + 50 }
      },
      React.createElement(List, {
        ref: 'List',
        className: styles.list,
        width: width,
        height: containerHeight,
        rowCount: years.length,
        rowHeight: rowHeight,
        scrollToIndex: selectedYearIndex + 1,
        scrollToAlignment: 'center',
        rowRenderer: function rowRenderer(_ref) {
          var index = _ref.index,
              rowStyle = _ref.style;

          var year = years[index];

          if (year !== null) {
            var _classNames;

            var isActive = index === selectedYearIndex;

            return React.createElement(
              'div',
              {
                key: index,
                className: classNames(styles.year, (_classNames = {}, _classNames[styles.active] = isActive, _classNames[styles.currentYear] = year === currentYear, _classNames)),
                onClick: function onClick() {
                  return _this2.handleClick(year);
                },
                title: 'Set year to ' + year,
                'data-year': year,
                style: Object.assign({}, rowStyle, {
                  color: typeof theme.selectionColor === 'function' ? theme.selectionColor(setYear(selectedDate, year)) : theme.selectionColor
                })
              },
              React.createElement(
                'span',
                {
                  style: year === currentYear ? { borderColor: theme.todayColor } : null
                },
                year
              )
            );
          } else {
            return React.createElement('div', { key: index, className: styles.spacer });
          }
        }
      })
    );
  };

  return Years;
}(Component);

export { Years as default };
process.env.NODE_ENV !== "production" ? Years.propTypes = {
  height: PropTypes.number,
  hideYearsOnSelect: PropTypes.bool,
  maxDate: PropTypes.object,
  minDate: PropTypes.object,
  onDaySelect: PropTypes.func,
  scrollToDate: PropTypes.func,
  selectedDate: PropTypes.object,
  setDisplay: PropTypes.func,
  theme: PropTypes.object,
  width: PropTypes.number,
  years: PropTypes.array
} : void 0;