var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _class, _temp;

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React, { Component, PropTypes } from 'react';
import classNames from 'classnames';
import debounce from 'lodash/debounce';
import range from 'lodash/range';
import { emptyFn, getScrollSpeed, keyCodes } from './utils';
import defaultDisplayOptions from './utils/defaultDisplayOptions';
import defaultLocale from './utils/defaultLocale';
import defaultTheme from './utils/defaultTheme';
import Today, { DIRECTION_UP, DIRECTION_DOWN } from './Today';
import Header from './Header';
import MonthList from './MonthList';
import Weekdays from './Weekdays';
import Years from './Years';

import parse from 'date-fns/parse';
import format from 'date-fns/format';
import addDays from 'date-fns/add_days';
import isSameDay from 'date-fns/is_same_day';
import isBefore from 'date-fns/is_before';
import isAfter from 'date-fns/is_after';
import getDay from 'date-fns/get_day';
import startOfMonth from 'date-fns/start_of_month';
import startOfDay from 'date-fns/start_of_day';

var styles = {
  container: {
    'root': 'Cal__Container__root',
    'landscape': 'Cal__Container__landscape',
    'wrapper': 'Cal__Container__wrapper',
    'listWrapper': 'Cal__Container__listWrapper'
  },
  day: {
    'root': 'Cal__Day__root',
    'enabled': 'Cal__Day__enabled',
    'highlighted': 'Cal__Day__highlighted',
    'today': 'Cal__Day__today',
    'disabled': 'Cal__Day__disabled',
    'selected': 'Cal__Day__selected',
    'month': 'Cal__Day__month',
    'year': 'Cal__Day__year',
    'selection': 'Cal__Day__selection',
    'day': 'Cal__Day__day'
  }
};

var InfiniteCalendar = (_temp = _class = function (_Component) {
  _inherits(InfiniteCalendar, _Component);

  function InfiniteCalendar(props) {
    _classCallCheck(this, InfiniteCalendar);

    var _this = _possibleConstructorReturn(this, _Component.call(this, props));

    _this.onDaySelect = function (selectedDate, e) {
      var shouldHeaderAnimate = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _this.state.shouldHeaderAnimate;
      var _this$props = _this.props,
          shouldPreventSelect = _this$props.shouldPreventSelect,
          onSelect = _this$props.onSelect;


      if (!shouldPreventSelect(selectedDate)) {
        onSelect(selectedDate, e);

        _this.setState({
          highlightedDate: new Date(selectedDate),
          selectedDate: selectedDate,
          shouldHeaderAnimate: shouldHeaderAnimate
        }, _this.clearHighlight);
      }
    };

    _this.getCurrentOffset = function () {
      return _this.scrollTop;
    };

    _this.getDateOffset = function (date) {
      return _this._MonthList && _this._MonthList.getDateOffset(date);
    };

    _this.scrollTo = function (offset) {
      return _this._MonthList && _this._MonthList.scrollTo(offset);
    };

    _this.scrollToDate = function () {
      var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new Date();
      var offset = arguments[1];

      return _this._MonthList && _this._MonthList.scrollToDate(date, offset);
    };

    _this.getScrollSpeed = getScrollSpeed();

    _this.onScroll = function (_ref) {
      var scrollTop = _ref.scrollTop;
      var onScroll = _this.props.onScroll;
      var isScrolling = _this.state.isScrolling;

      var _this$getDisplayOptio = _this.getDisplayOptions(),
          showTodayHelper = _this$getDisplayOptio.showTodayHelper,
          showOverlay = _this$getDisplayOptio.showOverlay;

      var scrollSpeed = _this.scrollSpeed = Math.abs(_this.getScrollSpeed(scrollTop));
      _this.scrollTop = scrollTop;

      // We only want to display the months overlay if the user is rapidly scrolling
      if (showOverlay && scrollSpeed >= 50 && !isScrolling) {
        _this.setState({
          isScrolling: true
        });
      }

      if (showTodayHelper) {
        _this.updateTodayHelperPosition(scrollSpeed);
      }

      onScroll(scrollTop);
      _this.onScrollEnd();
    };

    _this.onScrollEnd = debounce(function () {
      var onScrollEnd = _this.props.onScrollEnd;
      var isScrolling = _this.state.isScrolling;

      var _this$getDisplayOptio2 = _this.getDisplayOptions(),
          showTodayHelper = _this$getDisplayOptio2.showTodayHelper;

      if (isScrolling) {
        _this.setState({ isScrolling: false });
      }

      if (showTodayHelper) {
        _this.updateTodayHelperPosition(0);
      }

      onScrollEnd(_this.scrollTop);
    }, 150);

    _this.updateTodayHelperPosition = function (scrollSpeed) {
      var today = _this.today;
      var scrollTop = _this.scrollTop;
      var showToday = _this.state.showToday;
      var _this$props2 = _this.props,
          height = _this$props2.height,
          rowHeight = _this$props2.rowHeight;

      var _this$getDisplayOptio3 = _this.getDisplayOptions(),
          todayHelperRowOffset = _this$getDisplayOptio3.todayHelperRowOffset;

      var newState = void 0;

      if (!_this._todayOffset) {
        _this._todayOffset = _this.getDateOffset(today) + // scrollTop offset of the month "today" is in
        Math.ceil((today.getDate() - 7 + getDay(startOfMonth(today))) / 7) * rowHeight // offset of "today" within its month
        ;
      }

      if (scrollTop >= _this._todayOffset + rowHeight * (todayHelperRowOffset + 1)) {
        if (showToday !== DIRECTION_UP) newState = DIRECTION_UP; //today is above the fold
      } else if (scrollTop + height <= _this._todayOffset + rowHeight - rowHeight * (todayHelperRowOffset + 1)) {
        if (showToday !== DIRECTION_DOWN) newState = DIRECTION_DOWN; //today is below the fold
      } else if (showToday && scrollSpeed <= 1) {
        newState = false;
      }

      if (scrollTop === 0) {
        newState = false;
      }

      if (newState != null) {
        _this.setState({ showToday: newState });
      }
    };

    _this.handleKeyDown = function (e) {
      var _this$props3 = _this.props,
          maxDate = _this$props3.maxDate,
          minDate = _this$props3.minDate,
          onHighlightedDateChange = _this$props3.onHighlightedDateChange,
          onKeyDown = _this$props3.onKeyDown;
      var _this$state = _this.state,
          display = _this$state.display,
          selectedDate = _this$state.selectedDate,
          highlightedDate = _this$state.highlightedDate,
          showToday = _this$state.showToday;

      var delta = 0;

      onKeyDown(e);

      if ([keyCodes.left, keyCodes.up, keyCodes.right, keyCodes.down].indexOf(e.keyCode) > -1 && typeof e.preventDefault === 'function') {
        e.preventDefault();
      }

      if (!selectedDate) {
        selectedDate = new Date();
      }

      if (display === 'days') {
        if (!highlightedDate) {
          highlightedDate = new Date(selectedDate);
          _this.setState({ highlightedDate: highlightedDate });
        }

        switch (e.keyCode) {
          case keyCodes.enter:
            _this.onDaySelect(new Date(highlightedDate), e);
            return;
          case keyCodes.left:
            delta = -1;
            break;
          case keyCodes.right:
            delta = +1;
            break;
          case keyCodes.down:
            delta = +7;
            break;
          case keyCodes.up:
            delta = -7;
            break;
          default:
            delta = 0;
        }

        if (delta) {
          var rowHeight = _this.props.rowHeight;

          var newHighlightedDate = addDays(highlightedDate, delta);

          // Make sure the new highlighted date isn't before min / max
          if (isBefore(newHighlightedDate, minDate)) {
            newHighlightedDate = new Date(minDate);
          } else if (isAfter(newHighlightedDate, maxDate)) {
            newHighlightedDate = new Date(maxDate);
          }

          // Update the highlight indicator
          _this.clearHighlight();

          // Scroll the view
          if (!_this.currentOffset) _this.currentOffset = _this.getCurrentOffset();
          var currentOffset = _this.currentOffset;
          var monthOffset = _this.getDateOffset(newHighlightedDate);
          var navOffset = showToday ? 36 : 0;

          var highlightedEl = _this.highlightedEl = _this.node.querySelector('[data-date=\'' + format(newHighlightedDate, 'YYYY-MM-DD') + '\']');

          // Edge-case: if the user tries to use the keyboard when the new highlighted date isn't rendered because it's too far off-screen
          // We need to scroll to the month of the new highlighted date so it renders
          if (!highlightedEl) {
            _this.scrollTo(monthOffset - navOffset);
            return;
          }

          highlightedEl.classList.add(styles.day.highlighted);

          var dateOffset = highlightedEl.offsetTop - rowHeight;
          var newOffset = monthOffset + dateOffset;

          if (currentOffset !== newOffset) {
            _this.currentOffset = newOffset;
            _this.scrollTo(newOffset - navOffset);
          }

          // Update the reference to the currently highlighted date
          _this.setState({
            highlightedDate: newHighlightedDate
          }, function () {
            return onHighlightedDateChange(newHighlightedDate);
          });
        }
      } else if (display === 'years' && _this._Years) {
        _this._Years.handleKeyDown(e);
      }
    };

    _this.clearHighlight = function () {
      if (_this.highlightedEl) {
        _this.highlightedEl.classList.remove(styles.day.highlighted);
        _this.highlightedEl = null;
      }
    };

    _this.setDisplay = function (display) {
      _this.setState({ display: display });
    };

    var displayOptions = _this.getDisplayOptions(props.displayOptions);

    _this.updateYears(props);

    _this.state = {
      display: props.display,
      selectedDate: _this.parseSelectedDate(props.selectedDate),
      shouldHeaderAnimate: displayOptions.shouldHeaderAnimate
    };
    return _this;
  }

  InfiniteCalendar.prototype.componentDidMount = function componentDidMount() {
    var _props = this.props,
        autoFocus = _props.autoFocus,
        keyboardSupport = _props.keyboardSupport;


    if (keyboardSupport && autoFocus) {
      this.node.focus();
    }
  };

  InfiniteCalendar.prototype.componentWillReceiveProps = function componentWillReceiveProps(next) {
    var _props2 = this.props,
        min = _props2.min,
        minDate = _props2.minDate,
        max = _props2.max,
        maxDate = _props2.maxDate,
        selectedDate = _props2.selectedDate;
    var display = this.state.display;


    if (next.min !== min || next.minDate !== minDate || next.max !== max || next.maxDate !== maxDate) {
      this.updateYears(next);
    }
    if (next.selectedDate !== selectedDate) {
      var parsed = this.parseSelectedDate(next.selectedDate);
      this.setState({
        selectedDate: parsed
      });
      if (parsed) this.scrollToDate(parsed, -this.props.rowHeight * 2);
    } else if (next.minDate !== minDate || next.maxDate !== maxDate) {
      // Need to make sure the currently selected date is not before the new minDate or after maxDate
      var _selectedDate = this.parseSelectedDate(this.state.selectedDate);
      if (!isSameDay(_selectedDate, this.state.selectedDate)) {
        this.setState({
          selectedDate: _selectedDate
        });
      }
    }
    if (next.display !== display) {
      this.setState({
        display: next.display
      });
    }
  };

  InfiniteCalendar.prototype.parseSelectedDate = function parseSelectedDate(selectedDate) {
    if (!selectedDate) {
      return null;
    }

    selectedDate = parse(selectedDate);

    // Selected Date should not be before min date or after max date
    if (isBefore(selectedDate, this._minDate)) {
      return this._minDate;
    } else if (isAfter(selectedDate, this._maxDate)) {
      return this._maxDate;
    }

    return startOfDay(new Date(selectedDate));
  };

  InfiniteCalendar.prototype.updateYears = function updateYears() {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.props;

    this._min = parse(props.min);
    this._max = parse(props.max);
    this._minDate = parse(props.minDate);
    this._maxDate = parse(props.maxDate);

    var min = this._min.getFullYear();
    var max = this._max.getFullYear();

    var months = [];
    var year = void 0,
        month = void 0;
    for (year = min; year <= max; year++) {
      for (month = 0; month < 12; month++) {
        months.push({ month: month, year: year });
      }
    }

    this.months = months;
  };

  InfiniteCalendar.prototype.getDisabledDates = function getDisabledDates(disabledDates) {
    return disabledDates && disabledDates.map(function (date) {
      return format(parse(date), 'YYYY-MM-DD');
    });
  };

  InfiniteCalendar.prototype.getDisplayOptions = function getDisplayOptions() {
    var displayOptions = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.props.displayOptions;

    return Object.assign({}, defaultDisplayOptions, displayOptions);
  };

  InfiniteCalendar.prototype.getLocale = function getLocale() {
    return Object.assign({}, defaultLocale, this.props.locale);
  };

  InfiniteCalendar.prototype.getTheme = function getTheme() {
    return Object.assign({}, defaultTheme, this.props.theme);
  };

  InfiniteCalendar.prototype.sanitizeDate = function sanitizeDate(date) {
    var disabledDates = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.getDisabledDates();
    var disabledDays = this.props.disabledDays;


    if (!date) {
      return null;
    }

    // Selected date should not be disabled
    if (disabledDates && disabledDates.indexOf(format(date, 'YYYY-MM-DD')) !== -1 || disabledDays && disabledDays.indexOf(getDay(date)) !== -1) {
      return null;
    }

    return date;
  };

  InfiniteCalendar.prototype.render = function render() {
    var _classNames,
        _this2 = this;

    var _props3 = this.props,
        className = _props3.className,
        disabledDays = _props3.disabledDays,
        height = _props3.height,
        keyboardSupport = _props3.keyboardSupport,
        minDate = _props3.minDate,
        maxDate = _props3.maxDate,
        tabIndex = _props3.tabIndex,
        width = _props3.width,
        other = _objectWithoutProperties(_props3, ['className', 'disabledDays', 'height', 'keyboardSupport', 'minDate', 'maxDate', 'tabIndex', 'width']);

    var _getDisplayOptions = this.getDisplayOptions(),
        hideYearsOnSelect = _getDisplayOptions.hideYearsOnSelect,
        layout = _getDisplayOptions.layout,
        overscanMonthCount = _getDisplayOptions.overscanMonthCount,
        showHeader = _getDisplayOptions.showHeader,
        showOverlay = _getDisplayOptions.showOverlay,
        showTodayHelper = _getDisplayOptions.showTodayHelper;

    var _state = this.state,
        display = _state.display,
        isScrolling = _state.isScrolling,
        showToday = _state.showToday,
        shouldHeaderAnimate = _state.shouldHeaderAnimate;

    var disabledDates = this.getDisabledDates(this.props.disabledDates);
    var selectedDate = this.sanitizeDate(this.state.selectedDate, disabledDates);
    var locale = this.getLocale();
    var theme = this.getTheme();
    var today = this.today = startOfDay(new Date());

    return React.createElement(
      'div',
      {
        tabIndex: tabIndex,
        onKeyDown: keyboardSupport && this.handleKeyDown,
        className: classNames(className, styles.container.root, (_classNames = {}, _classNames[styles.container.landscape] = layout === 'landscape', _classNames)),
        style: { color: theme.textColor.default, width: width },
        'aria-label': 'Calendar',
        ref: function ref(node) {
          _this2.node = node;
        }
      },
      showHeader && React.createElement(Header, {
        selectedDate: selectedDate,
        shouldHeaderAnimate: shouldHeaderAnimate,
        layout: layout,
        theme: theme,
        locale: locale,
        scrollToDate: this.scrollToDate,
        setDisplay: this.setDisplay,
        display: display
      }),
      React.createElement(
        'div',
        { className: styles.container.wrapper },
        React.createElement(Weekdays, { weekdays: locale.weekdays, weekStartsOn: locale.weekStartsOn, theme: theme }),
        React.createElement(
          'div',
          { className: styles.container.listWrapper },
          showTodayHelper && React.createElement(Today, {
            scrollToDate: this.scrollToDate,
            show: showToday,
            today: today,
            theme: theme,
            todayLabel: locale.todayLabel.long
          }),
          React.createElement(MonthList, _extends({
            ref: function ref(instance) {
              _this2._MonthList = instance;
            }
          }, other, {
            width: width,
            height: height,
            selectedDate: selectedDate,
            disabledDates: disabledDates,
            disabledDays: disabledDays,
            months: this.months,
            onDaySelect: this.onDaySelect,
            onScroll: this.onScroll,
            isScrolling: isScrolling,
            today: today,
            min: this._min,
            minDate: this._minDate,
            maxDate: this._maxDate,
            theme: theme,
            locale: locale,
            overscanMonthCount: overscanMonthCount,
            showOverlay: showOverlay
          }))
        ),
        display === 'years' && React.createElement(Years, {
          ref: function ref(instance) {
            _this2._Years = instance;
          },
          width: width,
          height: height,
          onDaySelect: this.onDaySelect,
          minDate: minDate,
          maxDate: maxDate,
          selectedDate: selectedDate,
          theme: theme,
          today: today,
          years: range(this._min.getFullYear(), this._max.getFullYear() + 1),
          setDisplay: this.setDisplay,
          scrollToDate: this.scrollToDate,
          hideYearsOnSelect: hideYearsOnSelect
        })
      )
    );
  };

  return InfiniteCalendar;
}(Component), _class.defaultProps = {
  autoFocus: true,
  display: 'days',
  displayOptions: {},
  height: 500,
  keyboardSupport: true,
  max: new Date(2050, 11, 31),
  maxDate: new Date(2050, 11, 31),
  min: new Date(1980, 0, 1),
  minDate: new Date(1980, 0, 1),
  onHighlightedDateChange: emptyFn,
  onKeyDown: emptyFn,
  onScroll: emptyFn,
  onScrollEnd: emptyFn,
  onSelect: emptyFn,
  rowHeight: 56,
  selectedDate: new Date(),
  shouldPreventSelect: function shouldPreventSelect() {
    return false;
  },
  tabIndex: 1,
  width: 400
}, _temp);
export { InfiniteCalendar as default };
process.env.NODE_ENV !== "production" ? InfiniteCalendar.propTypes = {
  autoFocus: PropTypes.bool,
  className: PropTypes.string,
  disabledDates: PropTypes.arrayOf(PropTypes.instanceOf(Date)),
  disabledDays: PropTypes.arrayOf(PropTypes.number),
  display: PropTypes.oneOf(['years', 'days']),
  displayOptions: PropTypes.shape({
    hideYearsOnSelect: PropTypes.bool,
    layout: PropTypes.oneOf(['portrait', 'landscape']),
    overscanMonthCount: PropTypes.number,
    shouldHeaderAnimate: PropTypes.bool,
    showHeader: PropTypes.bool,
    showOverlay: PropTypes.bool,
    showTodayHelper: PropTypes.bool,
    todayHelperRowOffset: PropTypes.number
  }),
  height: PropTypes.number,
  keyboardSupport: PropTypes.bool,
  locale: PropTypes.shape({
    blank: PropTypes.string,
    headerFormat: PropTypes.string,
    todayLabel: PropTypes.shape({
      long: PropTypes.string,
      short: PropTypes.string
    }),
    weekdays: PropTypes.arrayOf(PropTypes.string),
    weekStartsOn: PropTypes.oneOf([0, 1, 2, 3, 4, 5, 6])
  }),
  max: PropTypes.instanceOf(Date),
  maxDate: PropTypes.instanceOf(Date),
  min: PropTypes.instanceOf(Date),
  minDate: PropTypes.instanceOf(Date),
  onHighlightedDateChange: PropTypes.func,
  onKeyDown: PropTypes.func,
  onScroll: PropTypes.func,
  onScrollEnd: PropTypes.func,
  onSelect: PropTypes.func,
  rowHeight: PropTypes.number,
  selectedDate: PropTypes.instanceOf(Date),
  shouldPreventSelect: PropTypes.func,
  tabIndex: PropTypes.number,
  theme: PropTypes.shape({
    floatingNav: PropTypes.shape({
      background: PropTypes.string,
      chevron: PropTypes.string,
      color: PropTypes.string
    }),
    headerColor: PropTypes.string,
    selectionColor: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
    textColor: PropTypes.shape({
      active: PropTypes.string,
      default: PropTypes.string
    }),
    todayColor: PropTypes.string,
    weekdayColor: PropTypes.string
  }),
  width: PropTypes.number
} : void 0;